@@ -55,7 +55,7 @@ def function_with_types_in_docstring(param1, param2):
 
     Args:
         param1 (int): The first parameter.
-        param2 (str): The second parameter.
+        param2 (str, {"html", "json", "xml"}, optional): The second parameter.
 
     Returns:
         bool: The return value. True for success, False otherwise.
