@@ -37,14 +37,16 @@ from configargparse import ConfigFileParserException, ConfigFileParser, Argument
 
 if sys.version_info >= (3, 11):
     from tomllib import load as _toml_load
-    import io
-    # The tomllib module from the standard library 
-    # expect a binary IO and will fail if receives otherwise. 
-    # So we hack a compat function that will work with TextIO and assume the utf-8 encoding.
-    def toml_load(stream: TextIO) -> Any:
-        return _toml_load(io.BytesIO(stream.read().encode()))
 else:
-    from toml import load as toml_load
+    from tomli import load as _toml_load
+
+import io
+# The tomllib module from the standard library
+# expect a binary IO and will fail if receives otherwise.
+# So we hack a compat function that will work with TextIO and assume the utf-8 encoding.
+def toml_load(stream: TextIO) -> Any:
+    return _toml_load(io.BytesIO(stream.read().encode()))
+
 
 # I did not invented these regex, just put together some stuff from:
 # - https://stackoverflow.com/questions/11859442/how-to-match-string-in-quotes-using-regex
