@@ -60,9 +60,6 @@ from pydoctor.epydoc.markup import Field, ObjClass, ParseError, ParsedDocstring,
 from pydoctor.epydoc.markup.plaintext import ParsedPlaintextDocstring
 from pydoctor.epydoc.docutils import new_document
 
-#: A dictionary whose keys are the "consolidated fields" that are
-#: recognized by epydoc; and whose values are the corresponding epydoc
-#: field names that should be used for the individual fields.
 CONSOLIDATED_FIELDS = {
     'parameters': 'param',
     'arguments': 'arg',
@@ -159,7 +156,7 @@ class ParsedRstDocstring(ParsedDocstring):
     def __repr__(self) -> str:
         return '<ParsedRstDocstring: ...>'
 
-class _EpydocReader(StandaloneReader):
+class _EpydocReader(StandaloneReader): # type:ignore[type-arg]
     """
     A reader that captures all errors that are generated by parsing,
     and appends them to a list as L{ParseError}.
@@ -169,14 +166,14 @@ class _EpydocReader(StandaloneReader):
         self._errors = errors
         StandaloneReader.__init__(self)
 
-    def get_transforms(self) -> List[Transform]:
+    def get_transforms(self) -> list[type[Transform]]:
         # Remove the DocInfo transform, to ensure that :author: fields
         # are correctly handled.
         return [t for t in StandaloneReader.get_transforms(self)
                 if t != frontmatter.DocInfo]
 
     def new_document(self) -> nodes.document:
-        document = new_document(self.source.source_path, self.settings)
+        document = new_document('docstring', self.settings)
         # Capture all warning messages.
         document.reporter.attach_observer(self.report)
         # Return the new document.
