@@ -1,3 +1,5 @@
+import docutils
+
 from pydoctor.test.epydoc.test_epytext2html import epytext2node
 
 def test_nested_markup() -> None:
@@ -8,7 +10,7 @@ def test_nested_markup() -> None:
         I{B{Inline markup} may be nested; and
         it may span} multiple lines.
         '''
-    expected = '''<document source="epytext">
+    expected = '''<document source="docstring">
     <paragraph>
         <emphasis>
             <strong>
@@ -22,10 +24,11 @@ def test_nested_markup() -> None:
     doc = '''
         It becomes a little bit complicated with U{B{custom} links <https://google.ca>}
         '''
-    expected = '''<document source="epytext">
+    docutils_0_22 = docutils.__version_info__ >= (0, 22)
+    expected = f'''<document source="docstring">
     <paragraph>
         It becomes a little bit complicated with 
-        <reference internal="False" refuri="https://google.ca">
+        <reference internal="{0 if docutils_0_22 else False}" refuri="https://google.ca">
             <strong>
                 custom
              links
@@ -36,7 +39,7 @@ def test_nested_markup() -> None:
     doc = '''
         It becomes a little bit complicated with L{B{custom} links <twisted.web()>}
         '''
-    expected = '''<document source="epytext">
+    expected = '''<document source="docstring">
     <paragraph>
         It becomes a little bit complicated with 
         <title_reference refuri="twisted.web">
